#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <usb.h>
#include <string.h>
#include <errno.h>

static struct usb_device *usbconsole_dev;
static struct usb_dev_handle *usbconsole_dev_handle;

#define VENDOR_ID              0x67b
#define PRODUCT_ID             0x0
#define BULK_ENDPOINT_IN       0x83
#define MAXPACKETSIZE          64
#define DEFAULT_TIMEOUT        1000

int main (int argc, char **argv) 
{
    int timeout, retval;
    struct usb_bus *bus, *busses;
    char *buffer;

    switch(argc) {
    case 1:
	timeout = DEFAULT_TIMEOUT;
	break;
    case 2:
	timeout = atoi(argv[1]);
	if (timeout <= 0) {
	    fprintf(stderr,"timeout: %d must be positive\n", 
		    timeout);
	    exit(1);
	}
	break;
    default:
	fprintf(stderr,"Usage: \"%s [timeout]\"\n", argv[0]);
	exit (1);
    }
	
    usb_init();
    usb_find_busses();
    usb_find_devices();

    usbconsole_dev = NULL;
    busses = usb_get_busses();
	
    buffer = malloc(MAXPACKETSIZE *sizeof(char));
    memset(buffer, 0 , MAXPACKETSIZE);
    for (bus = busses; bus; bus = bus->next) {
	struct usb_device *dev;
	for (dev = bus->devices; dev; dev=dev->next) {
	    if (dev->descriptor.idVendor == VENDOR_ID &&
		dev->descriptor.idProduct == PRODUCT_ID) 
		usbconsole_dev=dev;
	}
    }
    if (usbconsole_dev == NULL) {
	fprintf(stderr,"Cannot find usb PL2301 h2h cable\n");
	exit(1);
    }

    usbconsole_dev_handle = usb_open(usbconsole_dev);

    do {
	retval = usb_bulk_read(usbconsole_dev_handle,BULK_ENDPOINT_IN,
			       buffer,MAXPACKETSIZE,timeout);
	if (retval < 0) {
	    fprintf(stderr,"%s\n", strerror(errno));
	    exit(1);
	}
	write(STDOUT_FILENO,buffer,retval);
    } while(1);

    exit(0);
}
