#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <linux/usbdevice_fs.h>
#include <errno.h>
#include <signal.h>
#include <time.h>

#define VENDOR_ID        0x67b
#define PRODUCT_ID       0x0
#define BULK_ENDPOINT_IN 0x83
#define SIGUSB           40

/* Device descriptor */
struct usb_device_descriptor
{
    __u8  bLength;
    __u8  bDescriptorType;
    __u16 bcdUSB;
    __u8  bDeviceClass;
    __u8  bDeviceSubClass;
    __u8  bDeviceProtocol;
    __u8  bMaxPacketSize0;
    __u16 idVendor;
    __u16 idProduct;
    __u16 bcdDevice;
    __u8  iManufacturer;
    __u8  iProduct;
    __u8  iSerialNumber;
    __u8  bNumConfigurations;
} __attribute__ ((packed));



struct usb_device_descriptor usbconsole_dev;
struct usbdevfs_urb urb;

static char *buffer;
static int usbconsole_fd;

volatile int urb_completed = 0;

int usb_get_device_descriptor(char *path, __u16 idVendor, __u16 idProduct,
			      struct usb_device_descriptor *dev)
{
    int retval, fd;
    fd = open(path, O_RDWR);

    if (fd < 0) {
	return -1;
    }


    retval = read(fd, dev, sizeof(*dev));

    if ((dev->idVendor != idVendor) ||
	(dev->idProduct != idProduct)) {
	close(fd);
	fd = -1;
    }

    return fd;
}

int usb_claim_interface(int fd, int interface)
{
    return ioctl(fd, USBDEVFS_CLAIMINTERFACE, &interface);
}

int usb_submit_urb(int fd, struct usbdevfs_urb *urb)
{
    return ioctl(fd, USBDEVFS_SUBMITURB, urb);
}

void usb_fill_bulk_urb(struct usbdevfs_urb *urb, unsigned char endpoint,
		       void *buffer, int buffer_length, unsigned int signr)
{
    urb->type = USBDEVFS_URB_TYPE_BULK;
    urb->flags = USBDEVFS_URB_QUEUE_BULK;
    urb->endpoint = endpoint;
    urb->buffer = buffer;
    urb->buffer_length = buffer_length;
    urb->signr = signr;
}



void async_completion(int dummy)
{
    int retval;
    struct usbdevfs_urb *reaped_urb = &urb;
    retval = ioctl(usbconsole_fd, USBDEVFS_REAPURB, &reaped_urb);
    if (retval < 0) {
	fprintf(stderr,"reap:%s\n", strerror(errno));
    }
    write(STDOUT_FILENO, reaped_urb->buffer, reaped_urb->actual_length);
    fflush(stdout);
    urb_completed = 1;
}

int main (int argc, char **argv)
{
    int retval;
    struct timespec req;

    if (argc != 2) {
	fprintf(stderr,"Usage:\"%s <path>\"\n",
		argv[0]);
	exit(1);
    }
    usbconsole_fd = usb_get_device_descriptor(argv[1], VENDOR_ID, 
					      PRODUCT_ID,
					      &usbconsole_dev);
    if (usbconsole_fd == -1) {
	fprintf(stderr,"%s: %s: error while retrieving device ids\n", 
		argv[0], argv[1]);
	exit(1);
    }

    retval = usb_claim_interface(usbconsole_fd, 0);
    if (retval < 0) {
	fprintf(stderr,"while claiming interface: %s\n",
		strerror(errno));
	exit(1);
    }

    signal(SIGUSB, async_completion);
    urb.flags=0;

    buffer = malloc(sizeof(char) * 64);

    usb_fill_bulk_urb(&urb, BULK_ENDPOINT_IN, buffer, 64, SIGUSB);
    if (retval < 0) {
	fprintf(stderr,"%s\n",strerror(errno));
	exit(1);
    }
    req.tv_sec=0; req.tv_nsec=10000;
    retval = usb_submit_urb(usbconsole_fd, &urb);

    while(retval >= 0) 
    {
	nanosleep(&req, NULL);
	if (urb_completed) {
	    urb_completed = 0;
	    retval = usb_submit_urb(usbconsole_fd, &urb);
	    if (retval < 0) {
		fprintf(stderr,"%s\n",strerror(errno));
	    }
	}
    }

    exit(0);
}
